from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .mobile_app_identifier import MobileAppIdentifier

from .mobile_app_identifier import MobileAppIdentifier

@dataclass
class MacAppIdentifier(MobileAppIdentifier, Parsable):
    """
    The identifier for a Mac app.
    """
    # The OdataType property
    odata_type: Optional[str] = "#microsoft.graph.macAppIdentifier"
    # The identifier for an app, as specified in the app store.
    bundle_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MacAppIdentifier:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MacAppIdentifier
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MacAppIdentifier()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .mobile_app_identifier import MobileAppIdentifier

        from .mobile_app_identifier import MobileAppIdentifier

        fields: dict[str, Callable[[Any], None]] = {
            "bundleId": lambda n : setattr(self, 'bundle_id', n.get_str_value()),
        }
        super_fields = super().get_field_deserializers()
        fields.update(super_fields)
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        super().serialize(writer)
        writer.write_str_value("bundleId", self.bundle_id)
    

