from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .item_facet import ItemFacet
    from .locale_info import LocaleInfo

from .item_facet import ItemFacet

@dataclass
class UserAccountInformation(ItemFacet, Parsable):
    # The OdataType property
    odata_type: Optional[str] = "#microsoft.graph.userAccountInformation"
    # Shows the age group of user. Allowed values null, minor, notAdult and adult are generated by the directory and can't be changed.
    age_group: Optional[str] = None
    # Contains the two-character country code associated with the users' account.
    country_code: Optional[str] = None
    # The preferredLanguageTag property
    preferred_language_tag: Optional[LocaleInfo] = None
    # The user principal name (UPN) of the user associated with the account.
    user_principal_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> UserAccountInformation:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: UserAccountInformation
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return UserAccountInformation()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .item_facet import ItemFacet
        from .locale_info import LocaleInfo

        from .item_facet import ItemFacet
        from .locale_info import LocaleInfo

        fields: dict[str, Callable[[Any], None]] = {
            "ageGroup": lambda n : setattr(self, 'age_group', n.get_str_value()),
            "countryCode": lambda n : setattr(self, 'country_code', n.get_str_value()),
            "preferredLanguageTag": lambda n : setattr(self, 'preferred_language_tag', n.get_object_value(LocaleInfo)),
            "userPrincipalName": lambda n : setattr(self, 'user_principal_name', n.get_str_value()),
        }
        super_fields = super().get_field_deserializers()
        fields.update(super_fields)
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        super().serialize(writer)
        writer.write_str_value("ageGroup", self.age_group)
        writer.write_str_value("countryCode", self.country_code)
        writer.write_object_value("preferredLanguageTag", self.preferred_language_tag)
        writer.write_str_value("userPrincipalName", self.user_principal_name)
    

