from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import AdditionalDataHolder, Parsable, ParseNode, SerializationWriter
from kiota_abstractions.store import BackedModel, BackingStore, BackingStoreFactorySingleton
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class UserTrainingCompletionSummary(AdditionalDataHolder, BackedModel, Parsable):
    # Stores model information.
    backing_store: BackingStore = field(default_factory=BackingStoreFactorySingleton(backing_store_factory=None).backing_store_factory.create_backing_store, repr=False)

    # Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
    additional_data: dict[str, Any] = field(default_factory=dict)
    # The number of users who completed all the trainings before the due date.
    completed_users_count: Optional[int] = None
    # The number of users who started at least one training.
    in_progress_users_count: Optional[int] = None
    # The number of users who didn't complete all the trainings before the due date.
    not_completed_users_count: Optional[int] = None
    # The number of users who didn't start any training.
    not_started_users_count: Optional[int] = None
    # The OdataType property
    odata_type: Optional[str] = None
    # The number of users who are already assigned the same training.
    previously_assigned_users_count: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> UserTrainingCompletionSummary:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: UserTrainingCompletionSummary
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return UserTrainingCompletionSummary()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "completedUsersCount": lambda n : setattr(self, 'completed_users_count', n.get_int_value()),
            "inProgressUsersCount": lambda n : setattr(self, 'in_progress_users_count', n.get_int_value()),
            "notCompletedUsersCount": lambda n : setattr(self, 'not_completed_users_count', n.get_int_value()),
            "notStartedUsersCount": lambda n : setattr(self, 'not_started_users_count', n.get_int_value()),
            "@odata.type": lambda n : setattr(self, 'odata_type', n.get_str_value()),
            "previouslyAssignedUsersCount": lambda n : setattr(self, 'previously_assigned_users_count', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("completedUsersCount", self.completed_users_count)
        writer.write_int_value("inProgressUsersCount", self.in_progress_users_count)
        writer.write_int_value("notCompletedUsersCount", self.not_completed_users_count)
        writer.write_int_value("notStartedUsersCount", self.not_started_users_count)
        writer.write_str_value("@odata.type", self.odata_type)
        writer.write_int_value("previouslyAssignedUsersCount", self.previously_assigned_users_count)
        writer.write_additional_data_value(self.additional_data)
    

